from ...config import config
from ...utils import text_response, UserLocal
from ...command_matcher import MC
import tsugu_api
from ...utils import server_exists, server_name_2_server_id, server_id_2_server_name
from tsugu_api._typing import _ServerId, _Update
import typing
from ...storage.db import db_manager
import json


def handler(user: UserLocal, res: MC, platform: str, channel_id: str):
    server_mode: typing.Optional[_ServerId] = server_name_2_server_id(res.args[0]) if res.args else None
    if not server_mode:
        return text_response('未找到服务器，请输入正确的服务器名')
    cursor = db_manager.conn.cursor()
    cursor.execute("UPDATE users SET server_mode = ? WHERE user_id = ? AND platform = ?",
                   (server_mode, user.user_id, platform))
    db_manager.conn.commit()
    return text_response(f'服务器模式设置为 {server_id_2_server_name(server_mode)}')




from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api
from ...utils import server_exists, server_name_2_server_id


def handler(user: User, res: MC, platform: str, channel_id: str):
    if not res.args:
        return text_response('请输入正确的服务器名参数')
    server_pre = server_name_2_server_id(res.args[0])
    if not server_exists(server_pre):
        return text_response('未找到服务器，请输入正确的服务器名')
    player_id = int(user.server_list[server_pre]['playerId'])

    r = tsugu_api.bind_player_verification(platform, user.user_id, server_pre, player_id, False)
    if r.get('status') != 'success':
        return text_response(r.get('data'))
    return text_response('解绑成功')






from ...utils import text_response, server_exists
import tsugu_api
import time
from ...utils import User
from ...utils import config
from loguru import logger


def get_user(user_id: str, platform: str) -> User:
    for i in range(0, config.get_remote_user_data_max_retry):
        try:
            user_data_res = tsugu_api.get_user_data(platform, user_id)
            if user_data_res.get('status') == 'failed':
                return text_response(user_data_res.get('data'))
            break
        except Exception as e:
            logger.error(f'Error: {e}')
            time.sleep(0.5)
            continue
    else:
        return text_response('网络链接异常')

    # 获取用户数据失败
    if user_data_res.get('status') == 'failed':
        return text_response(user_data_res.get('data'))
    # 构建用户对象
    user_data = user_data_res.get('data')
    user = User(user_id, platform, user_data['server_mode'], user_data['default_server'], user_data['car'], user_data['server_list'])# type: ignore
    return user


import httpx

from ...config import config
from ...utils import text_response, UserLocal
from ...command_matcher import MC
# import tsugu_api
from ...utils import server_exists, server_name_2_server_id, server_id_2_server_name
import json
import urllib3
from ...storage.db import db_manager, get_user
from tsugu_api._typing import _ServerId


def get_bestdori_player(player_id: str, server: _ServerId):
    # 获取服务器名
    server_s_name = server_id_2_server_name(server)
    # 构建 URL
    url = f'https://bestdori.com/api/player/{server_s_name}/{player_id}?mode=2'
    if config.verify_player_bind_use_proxy:
        http = httpx.Client(proxies=config.proxy)
    else:
        http = httpx.Client()
    # 发送请求
    response = http.get(url)
    # 检查响应的状态码是否为 200
    if response.status_code == 200:
        # 解析 JSON 响应数据
        data = json.loads(response.text)
    else:
        return None
    return data


def bind_player_verification(platform: str, user: UserLocal, server: int | None, player_id: str, bind_type: bool):

    cursor = db_manager.conn.cursor()
    # 先检查verify_code是否正确
    # 使用 get
    verify_code = user.verify_code

    if server is None:
        server = user.server_mode
    if verify_code == "" or not verify_code:
        return {'status': 'error', 'data': '请先发送绑定请求'}
    # 检测重复性
    game_ids = user.game_ids
    for i in game_ids:
        if i.get("game_id") == player_id and i.get("server") == server:
            return {'status': 'error', 'data': '该记录已绑定'}

    # 获取玩家信息
    data = get_bestdori_player(player_id, server)

    if data.get("data").get("profile") is None or data.get("profile") == {}:
        return {'status': 'error', 'data': '未找到玩家，请检查玩家ID是否正确'}
    introduction = data.get("data", {}).get("profile", {}).get("introduction")
    deck_name = data.get("data", {}).get("profile", {}).get("mainUserDeck", {}).get("deckName")
    if verify_code != introduction and verify_code != deck_name:
        return {'status': 'error', 'data': '验证失败，请检查验证代码是否正确'}
    # 验证成功
    game_ids = user.game_ids
    game_ids.append({"game_id": player_id, "server": server})
    cursor.execute("UPDATE users SET game_ids = ? WHERE user_id = ? AND platform = ?",
                   (json.dumps(game_ids), user.user_id, platform))  # 存入game_ids
    cursor.execute("UPDATE users SET verify_code = ? WHERE user_id = ? AND platform = ?",
                   ("", user.user_id, platform))  # 清空verify_code
    db_manager.conn.commit()
    return {'status': 'success', 'data': '绑定成功! 现在可以使用"玩家状态"来查询玩家信息了'}


def handler(user: UserLocal, res: MC, platform: str, channel_id: str):
    if not res.args:
        return text_response('请输入正确的玩家ID和服务器名参数(可选)')
    if not res.args[0].isdigit():
        return text_response('请输入正确的玩家ID')
    player_id = int(res.args[0])
    if len(res.args) > 1:
        server_pre = server_name_2_server_id(res.args[1])
        if not server_exists(server_pre):
            return text_response('未找到服务器，请输入正确的服务器名')
        user.server_mode = server_pre

    r = bind_player_verification(platform, user, user.server_mode, player_id, True)
    if r.get('status') != 'success':
        return text_response(r.get('data'))
    return text_response('绑定成功')






from ...config import config
import tsugu_api
from ...utils import server_exists, server_name_2_server_id, server_id_2_server_name
from tsugu_api._typing import _ServerId, _Update

from ...utils import text_response, UserLocal
from ...command_matcher import MC



def handler(user: UserLocal, res: MC, platform: str, channel_id: str):
    # 无参数
    if not res.args:
        # 默认情况1: 优先当前服务器
        for i in user.game_ids:
            if i.get("server") == user.server_mode:
                player_id = str(i.get("game_id"))
                server = int(i.get("server"))
                text = f'已查找默认服务器 {server_id_2_server_name(server)} 的记录'
                msg = tsugu_api.search_player(int(player_id), server) + text_response(text)
                return msg
        else:
            # 默认情况2: 优先第一个记录
            if len(user.game_ids) > 0:
                player_id = str(user.game_ids[0].get("game_id"))
                server = int(user.game_ids[0].get("server"))
                text = f'已查找第一个记录 {player_id}'
                msg = [text_response(text), tsugu_api.search_player(int(player_id), server)]
                return msg
            else:
                return text_response('未绑定任何记录')

    # 查询指定服务器的玩家状态
    server: _ServerId = server_name_2_server_id(res.args[0])
    if server_exists(server):
        for i in user.game_ids:
            if i.get("server") == server:
                player_id = str(i.get("game_id"))
                text = f'已查找指定服务器 {server_id_2_server_name(server)} 的记录'
                msg = tsugu_api.search_player(int(player_id), server) + text_response(text)
                return msg
        else:
            return text_response(
                f'未在 {len(user.game_ids)} 条记录中找到 {server_id_2_server_name(server)} 的记录')

    # 查询指定记录顺序的玩家状态
    if res.args[0].isdigit():
        if int(res.args[0]) > len(user.game_ids) or int(res.args[0]) < 1:
            return text_response(f'未找到记录 {res.args[0]}')
        player_id = str(user.game_ids[int(res.args[0]) - 1].get("game_id"))
        server = int(user.game_ids[int(res.args[0]) - 1].get("server"))
        text = f'已查找第 {res.args[0]} 条记录'
        msg = tsugu_api.search_player(int(player_id), server) + text_response(text)
        return msg

    return text_response('请确保输入是 服务器名(字母缩写) 或者 记录(数字)')






from ...config import config
from ...utils import text_response, UserLocal
from ...command_matcher import MC
from ...utils import server_names_2_server_ids, server_ids_2_server_names
import json
from ...storage.db import db_manager


def handler(user: UserLocal, res: MC, platform: str, channel_id: str):
    default_server = server_names_2_server_ids(res.args)
    if not default_server:
        return text_response('未找到服务器，请输入正确的服务器名')
    cursor = db_manager.conn.cursor()
    cursor.execute("UPDATE users SET default_server = ? WHERE user_id = ? AND platform = ?",
                   (json.dumps(default_server), user.user_id, platform))
    db_manager.conn.commit()
    return text_response(f'默认服务器设置为 {server_ids_2_server_names(default_server)}')


import os
import sys
import sqlite3
from loguru import logger
import json
from ...utils import config
from ...utils import UserLocal


class DatabaseManager:
    def __init__(self, path):
        self.path = path
        self.conn = None
        self.cursor = None
        self.init_db(self.path)

    def init_db(self, path):
        if path:
            self.conn = sqlite3.connect(path, check_same_thread=False)
            logger.success(f'数据库连接成功，路径: {path}')
            self.cursor = self.conn.cursor()
            self.cursor.execute('''
                CREATE TABLE IF NOT EXISTS users (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    user_id TEXT UNIQUE,
                    platform TEXT,
                    server_mode INTEGER,
                    default_server TEXT,
                    car INTEGER,
                    game_ids TEXT,
                    verify_code TEXT
                )
            ''')
            self.conn.commit()
        else:
            # 如果不使用数据库，设置连接为 None
            self.conn = None
            # logger.info('不使用数据库')

    def close_db(self):
        if self.conn:
            self.conn.close()

    def execute_query(self, query):
        if self.cursor:
            self.cursor.execute(query)
            return self.cursor.fetchall()
        else:
            return []


db_manager = DatabaseManager(config._user_database_path)


def database(path: str = None):
    '''
    启用本地用户数据库，同时不再使用远程数据库
    本地数据库不存在时自动创建
    :param path: 数据库文件路径
    :return:
    '''
    if not path:
        path = (os.path.dirname(sys.modules['__main__'].__file__))
    config._user_database_path = path
    db_manager.init_db(path)


def get_user(platform: str, user_id: str):

    cursor = db_manager.conn.cursor()
    cursor.execute("SELECT * FROM users WHERE user_id = ? AND platform = ?", (user_id, platform))
    row = cursor.fetchone()

    if row:
        user_data = {
            "user_id": row[1],
            "platform": row[2],
            "server_mode": row[3],
            "default_server": list(json.loads(row[4])),
            "car": row[5],
            "game_ids": row[6],
            "verify_code": row[7]
        }
    else:
        user_data = {
            "user_id": user_id,
            "platform": platform,
            "server_mode": 3,
            "default_server": [3, 0],
            "car": 1,
            "game_ids": json.dumps([]),
            "verify_code": ""
        }
        cursor.execute('''
        INSERT INTO users (user_id, platform, server_mode, default_server, car, game_ids)
        VALUES (?, ?, ?, ?, ?, ?)
        ''', (user_id, platform, 3, json.dumps([3, 0]), 1, json.dumps([])))
        db_manager.conn.commit()

        game_ids = json.loads(user_data['game_ids'])
        user = UserLocal(user_id, platform, user_data['server_mode'], user_data['default_server'], user_data['car'], [],
                         game_ids, user_data['verify_code'])

        return user




from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api_async
from ...utils import server_exists, server_name_2_server_id, server_id_2_server_name


async def handler(user: User, res: MC, platform: str, channel_id: str):
    if res.args:
        server_pre = server_name_2_server_id(res.args[0])
        if not server_exists(server_pre):
            return text_response('未找到服务器，请输入正确的服务器名')
        user.server_mode = server_pre
    # 获取绑定请求
    r = await tsugu_api_async.bind_player_request(platform, user.user_id, user.server_mode, True)
    if r.get('status') != 'success':
        return text_response(r.get('data'))
    return text_response(
        f'''正在绑定账号，请将 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{r.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证 你的玩家ID(数字) {server_id_2_server_name(user.server_mode)}\n''')




from ...config import config
from ...utils import text_response, UserLocal
from ...command_matcher import MC
import tsugu_api
from ...utils import server_exists, server_name_2_server_id
from tsugu_api._typing import _ServerId

from ...storage.db import db_manager, get_user


def handler(user: UserLocal, res: MC, platform: str, channel_id: str):
    if not res.args:
        return text_response('请输入正确的记录(数字)\n例如: 解除绑定 1')

    if not res.args[0].isdigit():
        return text_response('请输入正确的记录(数字)')

    now_game_ids = user.game_ids
    # 检测是否存在该记录
    if int(res.args[0]) > len(now_game_ids):
        return text_response(f'未找到记录 {res.args[0]}')

    exe_game_id = user.game_ids.pop(int(res.args[0]) - 1)

    cursor = db_manager.conn.cursor()
    cursor.execute("UPDATE users SET game_ids = ? WHERE user_id = ? AND platform = ?",
                   (str(now_game_ids), user.user_id, platform))
    db_manager.conn.commit()

    return text_response(f'已解除绑定 {exe_game_id.get("game_id")}')




import tsugu_api
import tsugu_async
import unittest
import os
from PIL import Image
import io


class TestTsugu(unittest.IsolatedAsyncioTestCase):

    async def test_tsugu(self):
        async def show_back_msg(data):
            for i in data:
                if isinstance(i, str):
                    print(f"[文字信息]\n{i}")
                elif isinstance(i, bytes):
                    print(f"[图片信息]")
                    img = Image.open(io.BytesIO(i))
                    img.show()
                    print(f"[图像大小: {len(i) / 1024:.2f}KB]")

        # tsugu.database('./user_data.db')
        # tsugu_async.config.reload_from_json('./config.json')

        msg1 = await tsugu_async.handler('wdf', '1528593481', 'red', '666808414')
        await show_back_msg(msg1)

        # msg2 = await tsugu_async.handler('ycm', '1528593481', 'red', '666808414')
        # await show_back_msg(msg2)

        # msg3 = await tsugu_async.handler('12324 测q1试', '1528593481', 'red', '666808414')
        # await show_back_msg(msg3)

        # print(tsugu_api.get_user_data('red', '1528593481'))


if __name__ == '__main__':
    unittest.main()



from ..utils import server_exists, text_response, config, UserLocal

from ..universal import universal_api_handler
from ..utils import text_response, server_exists
from ..command_matcher import match_command
from . import router
from ..storage import db
from tsugu_api._typing import _ServerId
import json


def local_api_handler(user_id: str, message: str,  platform: str, channel_id: str):
    for i in config.user_commands:
        if res := match_command(message, commands=i['command_name']):
            user = db.get_user(user_id, platform)
            api = i['api']
            return router.api_handler(user, res, api, platform, channel_id)

    return None


def handler(message, user_id, platform, channel_id):
    # 进行 api 命令匹配
    result = universal_api_handler(user_id, message, platform, channel_id, local=True)
    if result:
        return result

    # 远程命令
    result = local_api_handler(user_id, message, platform, channel_id)
    if result:
        return result

    return None



from ...config import config
from ...utils import text_response, User, server_name_2_server_id, server_exists
from ...command_matcher import MC
import tsugu_api_async


async def handler(user: User, res: MC, platform: str, channel_id: str):
    if res.args:
        if server_exists(server_name_2_server_id(res.text)):
            user.server_mode = server_name_2_server_id(res.args[0])
    return await tsugu_api_async.song_meta(user.default_server, user.server_mode)

